@@ -3,10 +3,10 @@
                             -------------------
     begin                : Sat Jul 20 2002
     copyright            : (C) 2002      by André Somers,
-                               2008-2010 by Axel Pauli
+                               2008-2012 by Axel Pauli
     email                : axel@kflog.org
 
-    $Id: gpsnmea.h 4376 2010-11-03 19:07:56Z axel $
+    $Id: gpsnmea.h 5000 2012-03-14 10:55:56Z axel $
 
  ***************************************************************************
  *                                                                         *
@@ -18,6 +18,8 @@
  ***************************************************************************/
 
 /**
+ * \class GpsNmea
+ *
  * \author André Somers, Axel Pauli
  *
  * \brief NMEA parser, decoder and GPS connection handler.
@@ -25,6 +27,10 @@
  * This class parses and decodes the NMEA sentences and provides access
  * to the last know data. Furthermore it is managing the connection to a GPS
  * receiver connected by RS232, USB or to a Maemo GPS daemon process.
+ *
+ * \date 2002-2012
+ *
+ * \version $Id: gpsnmea.h 5000 2012-03-14 10:55:56Z axel $
  */
 
 #ifndef GPS_NMEA_H
@@ -37,12 +43,19 @@
 #include <QPoint>
 #include <QTimer>
 #include <QHash>
+#include <QFile>
+#include <QSet>
+#include <QMutex>
 
 #include "speed.h"
 #include "altitude.h"
-#include "gpscon.h"
 #include "wgspoint.h"
 
+#ifndef ANDROID
+#include "gpscon.h"
+#else
+#include <QEvent>
+#endif
 
 struct SatInfo
   {
@@ -157,14 +170,7 @@ class GpsNmea : public QObject
     void startGpsReceiver();
 
     /**
-     * This method can be used to read data from the connected GPS. It is provided
-     * to empty the receiver queue during long running other actions to avoid a
-     * buffer overflow.
-     */
-    void readDataFromGps();
-
-    /**
-     * @Returns the current GPS connection status.
+     * @return the current GPS connection status.
      */
     GpsNmea::GpsStatus getGpsStatus() const
       {
@@ -172,7 +178,7 @@ class GpsNmea : public QObject
       }
 
     /**
-     * @Returns the current GPS connection status. True if connected, false if not.
+     * @return the current GPS connection status. True if connected, false if not.
      */
     bool getConnected() const
       {
@@ -180,7 +186,7 @@ class GpsNmea : public QObject
       }
 
     /**
-     * @Returns the last known speed.
+     * @return the last known speed.
      */
     Speed getLastSpeed() const
       {
@@ -188,7 +194,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last known TAS.
+     * @return the last known TAS.
      */
     Speed getLastTas() const
       {
@@ -196,7 +202,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the date of the last fix.
+     * @return the date of the last fix.
      */
     QDate getLastDate() const
       {
@@ -204,7 +210,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the time of the last fix.
+     * @return the time of the last fix.
      */
     QTime getLastTime() const
       {
@@ -212,7 +218,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the date time as UTC of the last fix.
+     * @return the date time as UTC of the last fix.
      */
     QDateTime getLastUtc() const
       {
@@ -220,7 +226,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last known coordinate in KFLog format (x=lat, y=lon).
+     * @return the last known coordinate in KFLog format (x=lat, y=lon).
      */
     QPoint getLastCoord() const
       {
@@ -228,7 +234,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last know heading.
+     * @return the last know heading.
      */
     double getLastHeading() const
       {
@@ -236,7 +242,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last know standard pressure altitude
+     * @return the last know standard pressure altitude
      */
     Altitude getLastStdAltitude() const
       {
@@ -244,7 +250,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last know pressure altitude above sea level
+     * @return the last know pressure altitude above sea level
      */
     Altitude getLastPressureAltitude() const
       {
@@ -252,13 +258,13 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last know gps altitude depending on user
+     * @return the last know gps altitude depending on user
      * selection MSL or Pressure
      */
     Altitude getLastAltitude() const;
 
     /**
-     * @Returns the last know altitude above the WGS84 ellipsoid
+     * @return the last know altitude above the WGS84 ellipsoid
      */
     Altitude getLastGNSSAltitude() const
       {
@@ -266,7 +272,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last known wind speed.
+     * @return the last known wind speed.
      */
     Speed getLastWindSpeed() const
       {
@@ -274,7 +280,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last known wind direction.
+     * @return the last known wind direction.
      */
     short getLastWindDirection() const
       {
@@ -282,7 +288,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last known wind age in seconds.
+     * @return the last known wind age in seconds.
      */
     int getLastWindAge() const
       {
@@ -290,7 +296,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last known variometer speed.
+     * @return the last known variometer speed.
      */
     Speed getLastVariometer() const
       {
@@ -298,7 +304,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the last know satellite constellation string.
+     * @return the last know satellite constellation string.
      */
     SatInfo& getLastSatInfo()
       {
@@ -336,7 +342,7 @@ class GpsNmea : public QObject
     };
 
     /**
-     * @Returns selected altitude reference delivered by the GPS unit
+     * @return selected altitude reference delivered by the GPS unit
      */
     GpsNmea::DeliveredAltitude getDeliveredAltitude() const
       {
@@ -344,7 +350,7 @@ class GpsNmea : public QObject
       };
 
     /**
-     * @Returns the satellites in view.
+     * @return the satellites in view.
      */
     QList<SIVInfo>& getSivInfo()
     {
@@ -352,17 +358,21 @@ class GpsNmea : public QObject
     };
 
     /**
-     * @Returns the map datum of the GPS receiver.
+     * @return the map datum of the GPS receiver.
      */
     QString getMapDatum() const
       {
         return _mapDatum;
       };
 
-    static const QHash<QString, short>& getGpsMessageKeys()
-    {
-      return gpsHash;
-    };
+    /**
+     * Puts all desired GPS message keys into the passed hash. This method is
+     * thread safe.
+     *
+     * @param gpsKeys hash table where the results are stored.
+     *
+     */
+    static void getGpsMessageKeys( QHash<QString, short>& gpsKeys );
 
   public slots: // Public slots
     /**
@@ -379,6 +389,16 @@ class GpsNmea : public QObject
      */
     void slot_reset();
 
+    /**
+     * called to open the NMEA log file.
+     */
+    void slot_openNmeaLogFile();
+
+    /**
+     * called to close the NMEA log file.
+     */
+    void slot_closeNmeaLogFile();
+
 #if 0
     /**
      * This slot is called to reset the gps device to factory
@@ -395,6 +415,15 @@ class GpsNmea : public QObject
     void sendLastFix (bool hard, bool soft);
 #endif
 
+#ifdef ANDROID
+
+  protected:
+
+    /** Add an event receiver, used by Android only. */
+    bool event(QEvent *event);
+
+#endif
+
   private slots: // Private slots
 
     /** This slot is called by the external GPS receiver process to signal
@@ -537,11 +566,11 @@ class GpsNmea : public QObject
     void __ExtractPflau( const QStringList& slst );
 #endif
 
-    /** This function returns a QTime from the time encoded in a MNEA sentence. */
+    /** This function return a QTime from the time encoded in a MNEA sentence. */
     QTime __ExtractTime(const QString& timestring);
-    /** This function returns a QDate from the date encoded in a MNEA sentence. */
+    /** This function return a QDate from the date encoded in a MNEA sentence. */
     QDate __ExtractDate(const QString& datestring);
-    /** This function returns a Speed from the speed encoded in knots */
+    /** This function return a Speed from the speed encoded in knots */
     Speed __ExtractKnotSpeed(const QString& speedstring);
     /** This function converts the coordinate data from the NMEA sentence to the internal QPoint coordinate format. */
     QPoint __ExtractCoord(const QString& slat, const QString& slatNS, const QString& slon, const QString& slonEW);
@@ -592,7 +621,7 @@ class GpsNmea : public QObject
     /** This function calculates the checksum in the sentence. */
     static uint calcCheckSum (int pos, const QString& sentence);
     /** This function checks if the checksum in the sentence matches the sentence.
-     *  It returns true if it matches, and false otherwise. */
+     *  It return true if it matches, and false otherwise. */
     static bool checkCheckSum(int pos, const QString& sentence);
 
     /** This function calculates the STD altitude from the passed MSL altitude. */
@@ -677,8 +706,21 @@ class GpsNmea : public QObject
     QString _mapDatum;
     /** selected GPS device */
     QString gpsDevice;
-    /** reference to the normal serial connection */
+
+#ifndef ANDROID
+    /** The reference to the used serial connection */
     GpsCon* serial;
+#else
+    QObject* serial;
+#endif
+
+    /** Flag to enable/disable the GPS data processing. */
+    bool _enableGpsDataProcessing;
+
+    /** NMEA log file */
+    QFile* nmeaLogFile;
+    /** Flag to indicate receive of GPRMC. */
+    bool _gprmcSeen;
 
 #ifdef FLARM
 
@@ -693,6 +735,12 @@ class GpsNmea : public QObject
     // Dictionary with known sentence keywords
     static QHash<QString, short> gpsHash;
 
+    // Set with reported unknown GPS keys
+    QSet<QString> reportedUnknownKeys;
+
+    /** Mutex for thread synchronization. */
+    static QMutex mutex;
+
   public:
 
     // make class object for all available
