@@ -13,7 +13,7 @@
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: mapconfig.h 4709 2011-11-21 08:18:37Z axel $
+**   $Id: mapconfig.h 5024 2012-03-23 13:27:08Z axel $
 **
 ***********************************************************************/
 
@@ -29,8 +29,6 @@
 #include <QIcon>
 #include <QColor>
 
-#include "mapconfig.h"
-
 /**
  * \class MapConfig
  *
@@ -51,7 +49,7 @@
  *
  * \date 2001-2011
  *
- * \version $Id: mapconfig.h 4709 2011-11-21 08:18:37Z axel $
+ * \version $Id: mapconfig.h 5024 2012-03-23 13:27:08Z axel $
  */
 
 class MapConfig : public QObject
@@ -172,8 +170,6 @@ class MapConfig : public QObject
 
     /**
      * The possible data types, that could be drawn.
-     *
-     * @see #slotSetFlightDataType
      */
     bool isRotatable( unsigned int typeID ) const;
 
