@@ -7,17 +7,17 @@
  ************************************************************************
  **
  **   Copyright (c):  2000      by Heiner Lamprecht, Florian Ehinger
- **                   2008-2010 by Axel Pauli
+ **                   2008-2012 by Axel Pauli
  **
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: mapcontents.cpp 4155 2010-06-28 19:44:59Z axel $
+ **   $Id: mapcontents.cpp 4979 2012-03-07 13:19:28Z axel $
  **
  ***********************************************************************/
 
 #include <cmath>
-#include <stdlib.h>
+#include <cstdlib>
 #include <unistd.h>
 
 #include <QtGui>
@@ -26,6 +26,7 @@
 #include "airspace.h"
 #include "basemapelement.h"
 #include "calculator.h"
+#include "datatypes.h"
 #include "filetools.h"
 #include "flighttask.h"
 #include "generalconfig.h"
@@ -33,7 +34,7 @@
 #include "hwinfo.h"
 #include "isohypse.h"
 #include "lineelement.h"
-#include "map.h"
+#include "mainwindow.h"
 #include "mapcalc.h"
 #include "mapcontents.h"
 #include "mapmatrix.h"
@@ -74,9 +75,9 @@ extern MapView* _globalMapView;
 #define FILE_VERSION_MAP      101
 
 // compiled version
-#define FILE_VERSION_GROUND_C   103
-#define FILE_VERSION_TERRAIN_C  103
-#define FILE_VERSION_MAP_C      102
+#define FILE_VERSION_GROUND_C   104
+#define FILE_VERSION_TERRAIN_C  104
+#define FILE_VERSION_MAP_C      103
 
 
 #define READ_POINT_LIST\
@@ -108,7 +109,8 @@ const short MapContents::isoLevels[] =
 
 MapContents::MapContents(QObject* parent, WaitScreen* waitscreen) :
     QObject(parent),
-    isFirst(true)
+    isFirst(true),
+    isReload(false)
 #ifdef INTERNET
 
     , downloadManger(0),
@@ -131,17 +133,35 @@ MapContents::MapContents(QObject* parent, WaitScreen* waitscreen) :
   _isoLevelReset=true;
   _lastIsoEntry=0;
 
-  // read in waypoint list
+  // read in waypoint list from catalog
   WaypointCatalog wpCat;
-  wpCat.read( 0, wpList );
+  int ok;
+  const char* format;
+  QString error;
 
-  currentTask=0;
+  if( GeneralConfig::instance()->getWaypointFileFormat() == GeneralConfig::Binary )
+    {
+      ok = wpCat.readBinary( "", &wpList );
+      format = "binary";
+    }
+  else
+    {
+      ok = wpCat.readXml( "", &wpList, error );
+      format = "xml";
+    }
 
-  connect( this, SIGNAL(progress(int)),
-          ws, SLOT(slot_Progress(int)) );
+  if( ok )
+    {
+      qDebug() << "MapContents():" << wpList.size() << "waypoints read from"
+               << format << "catalog.";
+    }
+
+  currentTask = 0;
+
+  connect( this, SIGNAL(progress(int)), ws, SLOT(slot_Progress(int)) );
 
   connect( this, SIGNAL(loadingFile(const QString&)),
-          ws, SLOT(slot_SetText2(const QString&)) );
+           ws, SLOT(slot_SetText2(const QString&)) );
 
   // qDebug("MapContents initialized");
 }
@@ -153,14 +173,22 @@ MapContents::~MapContents()
       delete currentTask;
     }
 
-  qDeleteAll (airspaceList);
+  qDeleteAll(airspaceList);
 }
 
 // save the current waypoint list
 void MapContents::saveWaypointList()
 {
   WaypointCatalog wpCat;
-  wpCat.write( 0, wpList );
+
+  if( GeneralConfig::instance()->getWaypointFileFormat() == GeneralConfig::Binary )
+    {
+      wpCat.writeBinary( "", wpList );
+    }
+  else
+    {
+      wpCat.writeXml( "", wpList );
+    }
 }
 
 /**
@@ -319,7 +347,15 @@ bool MapContents::__readTerrainFile( const int fileSecID,
   emit loadingFile(pathName);
 
   QDataStream in(&mapfile);
-  in.setVersion(QDataStream::Qt_3_3);
+
+  if( compiling )
+    {
+      in.setVersion(QDataStream::Qt_3_3);
+    }
+  else
+    {
+      in.setVersion(QDataStream::Qt_4_7);
+    }
 
   // qDebug("reading file %s", pathName.toLatin1().data());
 
@@ -499,10 +535,10 @@ bool MapContents::__readTerrainFile( const int fileSecID,
   QFile ausgabe(kfcPathName);
   QDataStream out(&ausgabe);
 
-  if ( compiling )
+  if( compiling )
     {
       out.setDevice(&ausgabe);
-      out.setVersion(QDataStream::Qt_3_3);
+      out.setVersion(QDataStream::Qt_4_7);
 
       if (!ausgabe.open(QIODevice::WriteOnly))
         {
@@ -775,7 +811,15 @@ bool MapContents::__readBinaryFile(const int fileSecID,
   emit loadingFile(pathName);
 
   QDataStream in(&mapfile);
-  in.setVersion(QDataStream::Qt_2_0);
+
+  if( compiling )
+    {
+      in.setVersion( QDataStream::Qt_2_0 );
+    }
+  else
+    {
+      in.setVersion( QDataStream::Qt_4_7 );
+    }
 
   // qDebug("reading file %s", pathName.toLatin1().data());
 
@@ -970,8 +1014,9 @@ bool MapContents::__readBinaryFile(const int fileSecID,
 
   if ( compiling )
     {
-      out.setDevice(&ausgabe);
-      out.setVersion(QDataStream::Qt_2_0);
+      out.setDevice( &ausgabe );
+      out.setVersion( QDataStream::Qt_4_7 );
+
       if (!ausgabe.open(QIODevice::WriteOnly))
         {
           qWarning("Cumulus: Can't open compiled map file %s for writing!"
@@ -1020,12 +1065,12 @@ bool MapContents::__readBinaryFile(const int fileSecID,
 
       switch (typeIn)
         {
-        case BaseMapElement::Highway:
+        case BaseMapElement::Motorway:
           READ_POINT_LIST
 
-          if ( !GeneralConfig::instance()->getMapLoadHighways() ) break;
+          if ( !GeneralConfig::instance()->getMapLoadMotorways() ) break;
 
-          highwayList.append( LineElement("", typeIn, all, false, fileSecID) );
+          motorwayList.append( LineElement("", typeIn, all, false, fileSecID) );
           break;
 
         case BaseMapElement::Road:
@@ -1189,8 +1234,15 @@ bool MapContents::__readBinaryFile(const int fileSecID,
               in >> single;
             }
 
-          villageList.append( SinglePoint(name, "", typeIn,
-                                          WGSPoint(lat_temp, lon_temp), single, 0, fileSecID));
+          villageList.append( SinglePoint( name,
+                                           "",
+                                           typeIn,
+                                           WGSPoint(lat_temp, lon_temp),
+                                           single,
+                                           0,
+                                           "",
+                                           "",
+                                           fileSecID ) );
           // qDebug("added village '%s'", name.toLatin1().data());
           break;
 
@@ -1214,10 +1266,19 @@ bool MapContents::__readBinaryFile(const int fileSecID,
               out << single;
             }
           else
-            in >> single;
+            {
+              in >> single;
+            }
 
-          obstacleList.append( SinglePoint("Spot", "", typeIn,
-                                           WGSPoint(lat_temp, lon_temp), single, 0, fileSecID));
+          obstacleList.append( SinglePoint( "Spot",
+                                            "",
+                                            typeIn,
+                                            WGSPoint(lat_temp, lon_temp),
+                                            single,
+                                            0,
+                                            "",
+                                            "",
+                                            fileSecID ) );
           break;
 
         case BaseMapElement::Landmark:
@@ -1248,13 +1309,25 @@ bool MapContents::__readBinaryFile(const int fileSecID,
               out << single;
             }
           else
-            in >> single;
-          landmarkList.append( SinglePoint(name, "", typeIn,
-                                           WGSPoint(lat_temp, lon_temp), single, 0, fileSecID));
+            {
+              in >> single;
+            }
+
+          landmarkList.append( SinglePoint( name,
+                               "",
+                               typeIn,
+                               WGSPoint(lat_temp, lon_temp),
+                               single,
+                               0,
+                               "",
+                               "",
+                               fileSecID ) );
+
           // qDebug("added landmark '%s'", name.toLatin1().data());
           break;
         default:
           qWarning ("MapContents::__readBinaryFile; type not handled in switch: %d", typeIn);
+          break;
         }
 
       // @AP: Performance brake! emit progress calls waitscreen and
@@ -1311,8 +1384,8 @@ void MapContents::slotDownloadMapArea( const QPoint &center, const Distance& len
 
   // Calculate length in degree along the latitude and the longitude.
   // For the calculation the circle formula  is used.
-  double deltaLat = 180/M_PIl * radius/RADIUS;
-  double deltaLon = 180/M_PIl * radius/(RADIUS * cos ( M_PIl / 180.0 * centerLat ));
+  double deltaLat = 180/M_PI * radius/RADIUS;
+  double deltaLon = 180/M_PI * radius/(RADIUS * cos ( M_PI / 180.0 * centerLat ));
 
   // Calculate the sky boarders of the square.
   int north = (int) ceil(centerLat + deltaLat);
@@ -1516,12 +1589,24 @@ void MapContents::slotDownloadsFinished( int requests, int errors )
   // initiate a map redraw
   Map::instance->scheduleRedraw( Map::baseLayer );
 
-  QString msg;
-  msg = QString(tr("%1 download(s) with %2 error(s) done.")).arg(requests).arg(errors);
+  QString msg = QString(tr("%1 download(s) with %2 error(s) done.")).arg(requests).arg(errors);
 
-  QMessageBox::information( Map::instance,
-                            tr("Downloads finished"),
-                            msg );
+  QMessageBox mb( QMessageBox::Information,
+                  tr("Downloads finished"),
+                  msg,
+                  QMessageBox::Ok,
+                  MainWindow::mainWindow() );
+
+#ifdef ANDROID
+
+  mb.show();
+  QPoint pos = MainWindow::mainWindow()->mapToGlobal(QPoint( MainWindow::mainWindow()->width()/2  - mb.width()/2,
+                                                             MainWindow::mainWindow()->height()/2 - mb.height()/2 ));
+  mb.move( pos );
+
+#endif
+
+  mb.exec();
 }
 
 /** Called, if a network error occurred during the downloads. */
@@ -1534,12 +1619,24 @@ void MapContents::slotNetworkError()
   // Reset user decision flag
   shallDownloadData = false;
 
-  QString msg;
-  msg = QString(tr("Network error occurred.\nAll downloads are canceled!"));
+  QString msg = QString(tr("Network error occurred.\nAll downloads are canceled!"));
 
-  QMessageBox::information( Map::instance,
-                            tr("Network Error"),
-                            msg );
+  QMessageBox mb( QMessageBox::Warning,
+                  tr("Network Error"),
+                  msg,
+                  QMessageBox::Ok,
+                  MainWindow::mainWindow() );
+
+#ifdef ANDROID
+
+  mb.show();
+  QPoint pos = MainWindow::mainWindow()->mapToGlobal(QPoint( MainWindow::mainWindow()->width()/2  - mb.width()/2,
+                                                             MainWindow::mainWindow()->height()/2 - mb.height()/2 ));
+  mb.move( pos );
+
+#endif
+
+  mb.exec();
 }
 
 /**
@@ -1556,14 +1653,25 @@ bool MapContents::__askUserForDownload()
 
   hasAskForDownload = true;
 
-  int answer = QMessageBox::question( Map::instance,
-                  tr("Download missing Data?"),
-                  tr("Download missing Data from the Internet?") +
-                  QString("<p>") +
-                  tr("Active Internet connection is needed!"),
-                  QMessageBox::Yes | QMessageBox::No, QMessageBox::No );
+  QMessageBox mb( QMessageBox::Question,
+                  tr( "Download missing Data?" ),
+                  tr( "Download missing Data from the Internet?" ) +
+                  QString("<p>") + tr("Active Internet connection is needed!"),
+                  QMessageBox::Yes | QMessageBox::No,
+                  MainWindow::mainWindow() );
 
-  if( answer == QMessageBox::Yes )
+  mb.setDefaultButton( QMessageBox::No );
+
+#ifdef ANDROID
+
+  mb.show();
+  QPoint pos = MainWindow::mainWindow()->mapToGlobal(QPoint( MainWindow::mainWindow()->width()/2  - mb.width()/2,
+                                                             MainWindow::mainWindow()->height()/2 - mb.height()/2 ));
+  mb.move( pos );
+
+#endif
+
+  if( mb.exec() == QMessageBox::Yes )
     {
       shallDownloadData = true;
     }
@@ -1585,7 +1693,7 @@ void MapContents::proofeSection()
   // calling of this method
   static bool mutex = false;
 
-  if ( mutex )
+  if( mutex )
     {
       // qDebug("MapContents::proofeSection(): is recursive called, returning");
       return; // return immediately, if reenter in method is not possible
@@ -1615,68 +1723,46 @@ void MapContents::proofeSection()
   // qDebug( "MapBorderCorners: l=%d, r=%d, t=%d, b=%d",
   //          westCorner, eastCorner, northCorner, southCorner );
 
-  if (isFirst)
-    {
-      ws->slot_SetText1(tr("Loading maps..."));
-      ws->slot_SetText2(tr("Reading OpenAir Files"));
-
-      OpenAirParser oap;
-      oap.load( airspaceList );
-
-      // finally, sort the airspaces
-      airspaceList.sort();
-
-      ws->slot_SetText2(tr("Reading Welt2000 File"));
-      // @AP: Look for and if available load a welt2000 airfield file
-      Welt2000 welt2000;
-
-      if( ! welt2000.load( airfieldList, gliderSiteList, outLandingList ) )
-        {
-
-#ifdef INTERNET
-
-          if( __askUserForDownload() == true )
-            {
-              // Welt2000 load failed, try to download a new Welt2000 File.
-              slotDownloadWelt2000( GeneralConfig::instance()->getWelt2000FileName() );
-            }
-
-#endif
-
-        }
-    }
-
   unloadDone = false;
   memoryFull = false;
-  char step, hasstep; //used as small integers
+  char step, hasstep; // used as small integers
   TilePartMap::Iterator it;
 
-  for ( int row = northCorner; row <= southCorner; row++ )
+  if( isReload )
     {
-      for (int col = westCorner; col <= eastCorner; col++)
+      ws->setScreenUsage( true );
+      ws->setVisible( true );
+      QCoreApplication::processEvents( QEventLoop::ExcludeUserInputEvents |
+                                       QEventLoop::ExcludeSocketNotifiers );
+    }
+
+  if( isFirst )
+    {
+      ws->slot_SetText1( tr( "Loading maps..." ) );
+    }
+
+  for( int row = northCorner; row <= southCorner; row++ )
+    {
+      for( int col = westCorner; col <= eastCorner; col++ )
         {
           int secID = row + (col + (row * 179));
           // qDebug( "Needed BoxSecID=%d", secID );
 
-          if (isFirst)
+          if( isFirst )
             {
               // Animate a little bit during first load. Later on in flight,
               // we need the time for GPS processing.
-              emit progress(2);
+              emit progress( 2 );
             }
 
-          if ( (secID >= 0) && (secID <= MAX_TILE_NUMBER) )
+          if( secID >= 0 && secID <= MAX_TILE_NUMBER )
             {
               // a valid tile (2x2 degree area) must be in the range 0 ... 16200
-              if (!tileSectionSet.contains(secID))
+              if( ! tileSectionSet.contains( secID ) )
                 {
                   // qDebug(" Tile %d is missing", secID );
                   // Tile is missing
-                  if (isFirst)
-                    {
-                      ws->slot_SetText1(tr("Loading maps..."));
-                    }
-                  else
+                  if( ! isFirst)
                     {
                       // @AP: remove of all unused maps to get place
                       // in heap. That can be disabled here because
@@ -1685,7 +1771,7 @@ void MapContents::proofeSection()
                       // method is necessary. But the disadvantage
                       // is in that case that the freeing needs a
                       // lot of time (several seconds).
-                      if ( GeneralConfig::instance()->getMapUnload() )
+                      if( GeneralConfig::instance()->getMapUnload() )
                         {
                           unloadMaps(0);
                         }
@@ -1743,13 +1829,64 @@ void MapContents::proofeSection()
         }
     }
 
-  if ( isFirst )
+  if( isFirst )
     {
+      ws->slot_SetText2( tr( "Reading OpenAir Files" ) );
+
+      OpenAirParser oap;
+      oap.load( airspaceList );
+
+      // finally, sort the airspaces
+      airspaceList.sort();
+
+      ws->slot_SetText2( tr( "Reading Welt2000 File" ) );
+
+      if( isReload == false )
+        {
+          // Load airfield data not in an extra thread
+          Welt2000 welt2000;
+
+          if( ! welt2000.load( airfieldList, gliderfieldList, outLandingList ) )
+            {
+
+#ifdef INTERNET
+
+              if( __askUserForDownload() == true )
+                {
+                  // Welt2000 load failed, try to download a new Welt2000 File.
+                  slotDownloadWelt2000( GeneralConfig::instance()->getWelt2000FileName() );
+                }
+#endif
+            }
+        }
+      else
+        {
+
+#ifdef WELT2000_THREAD
+          // In case of an reload we assume, a Welt2000 file is available.
+          // Therefore the reload is done in an extra thread because it can
+          // take a while and the GUI shall not be blocked by this action.
+          loadWelt2000DataViaThread();
+#else
+          // Old solution. Should be used, if not enough RAM is available
+          // because thread loading needs temporary more memory for parallel load.
+          Welt2000 welt2000;
+          welt2000.load( airfieldList, gliderfieldList, outLandingList );
+#endif
+        }
+
       ws->slot_SetText1(tr("Loading maps done"));
     }
 
-  isFirst = false;
-  mutex = false; // unlock mutex
+  if( isReload )
+    {
+      ws->setScreenUsage( false );
+      ws->setVisible( false );
+    }
+
+  isFirst  = false;
+  isReload = false;
+  mutex    = false; // unlock mutex
 }
 
 // Distance unit is expected as meters. The bounding map rectangle will be
@@ -1758,9 +1895,9 @@ void MapContents::unloadMaps(unsigned int distance)
 {
   // qDebug("MapContents::unloadMaps() is called");
 
-  if (unloadDone)
+  if( unloadDone )
     {
-      return;  //we only unload map data once (per map redrawing round)
+      return; // we only unload map data once (per map redrawing round)
     }
 
   extern MapMatrix* _globalMapMatrix;
@@ -1902,11 +2039,11 @@ void MapContents::unloadMaps(unsigned int distance)
   qDebug("Unload reportList(%d), elapsed=%d", reportList.count(), t.restart());
 #endif
 
-  unloadMapObjects( highwayList );
+  unloadMapObjects( motorwayList );
 
 #ifdef DEBUG_UNLOAD
   sum += t.elapsed();
-  qDebug("Unload highwayList(%d), elapsed=%d", highwayList.count(), t.restart());
+  qDebug("Unload motorwayList(%d), elapsed=%d", motorwayList.count(), t.restart());
 #endif
 
   unloadMapObjects( roadList );
@@ -2035,8 +2172,8 @@ void MapContents::clearList(const int listIndex)
     case AirfieldList:
       airfieldList.clear();
       break;
-    case GliderSiteList:
-      gliderSiteList.clear();
+    case GliderfieldList:
+      gliderfieldList.clear();
       break;
     case OutLandingList:
       outLandingList.clear();
@@ -2062,8 +2199,8 @@ void MapContents::clearList(const int listIndex)
     case LandmarkList:
       landmarkList.clear();
       break;
-    case HighwayList:
-      highwayList.clear();
+    case MotorwayList:
+      motorwayList.clear();
       break;
     case RoadList:
       roadList.clear();
@@ -2093,8 +2230,8 @@ unsigned int MapContents::getListLength( const int listIndex ) const
     {
     case AirfieldList:
       return airfieldList.count();
-    case GliderSiteList:
-      return gliderSiteList.count();
+    case GliderfieldList:
+      return gliderfieldList.count();
     case OutLandingList:
       return outLandingList.count();
     case RadioList:
@@ -2111,8 +2248,8 @@ unsigned int MapContents::getListLength( const int listIndex ) const
       return villageList.count();
     case LandmarkList:
       return landmarkList.count();
-    case HighwayList:
-      return highwayList.count();
+    case MotorwayList:
+      return motorwayList.count();
     case RoadList:
       return roadList.count();
     case RailList:
@@ -2133,14 +2270,14 @@ Airspace* MapContents::getAirspace(unsigned int index)
   return static_cast<Airspace *> (airspaceList[index]);
 }
 
-Airfield* MapContents::getAirport(unsigned int index)
+Airfield* MapContents::getAirfield(unsigned int index)
 {
   return &airfieldList[index];
 }
 
-Airfield* MapContents::getGlidersite(unsigned int index)
+Airfield* MapContents::getGliderfield(unsigned int index)
 {
-  return &gliderSiteList[index];
+  return &gliderfieldList[index];
 }
 
 Airfield* MapContents::getOutlanding(unsigned int index)
@@ -2154,8 +2291,8 @@ BaseMapElement* MapContents::getElement(int listType, unsigned int index)
     {
     case AirfieldList:
       return &airfieldList[index];
-    case GliderSiteList:
-      return &gliderSiteList[index];
+    case GliderfieldList:
+      return &gliderfieldList[index];
     case OutLandingList:
       return &outLandingList[index];
     case RadioList:
@@ -2172,8 +2309,8 @@ BaseMapElement* MapContents::getElement(int listType, unsigned int index)
       return &villageList[index];
     case LandmarkList:
       return &landmarkList[index];
-    case HighwayList:
-      return &highwayList[index];
+    case MotorwayList:
+      return &motorwayList[index];
     case RoadList:
       return &roadList[index];
     case RailList:
@@ -2197,8 +2334,8 @@ SinglePoint* MapContents::getSinglePoint(int listIndex, unsigned int index)
     {
     case AirfieldList:
       return static_cast<SinglePoint *> (&airfieldList[index]);
-    case GliderSiteList:
-      return static_cast<SinglePoint *> (&gliderSiteList[index]);
+    case GliderfieldList:
+      return static_cast<SinglePoint *> (&gliderfieldList[index]);
     case OutLandingList:
       return static_cast<SinglePoint *> (&outLandingList[index]);
     case RadioList:
@@ -2252,19 +2389,23 @@ void MapContents::slotReloadMapData()
   obstacleList.clear();
   railList.clear();
   reportList.clear();
-  highwayList.clear();
+  motorwayList.clear();
   roadList.clear();
   topoList.clear();
   villageList.clear();
 
+  welt2000Mutex.lock();
+
   airfieldList.clear();
-  gliderSiteList.clear();
+  gliderfieldList.clear();
   outLandingList.clear();
 
   // free internal allocated memory in QList
-  airfieldList   = QList<Airfield>();
-  gliderSiteList = QList<Airfield>();
-  outLandingList = QList<Airfield>();
+  airfieldList    = QList<Airfield>();
+  gliderfieldList = QList<Airfield>();
+  outLandingList  = QList<Airfield>();
+
+  welt2000Mutex.unlock();
 
   // all isolines are cleared
   groundMap.clear();
@@ -2274,9 +2415,10 @@ void MapContents::slotReloadMapData()
   tileSectionSet.clear();
   tilePartMap.clear();
 
-  isFirst = true;
+  isFirst  = true;
+  isReload = true;
 
-  // @AP: Reload all data, that must be always done after a projection data change
+  // Reload all data, that must be always done after a projection data change
   proofeSection();
 
   // Check for a selected waypoint, this one must be also new projected.
@@ -2285,7 +2427,7 @@ void MapContents::slotReloadMapData()
   extern MapMatrix   *_globalMapMatrix;
 
   // Update the global selected waypoint
-  wayPoint *wp = (wayPoint *) calculator->getselectedWp();
+  Waypoint *wp = (Waypoint *) calculator->getselectedWp();
 
   if ( wp )
     {
@@ -2321,11 +2463,87 @@ void MapContents::slotReloadMapData()
   mutex = false; // unlock mutex
 }
 
+#ifdef WELT2000_THREAD
+
 /**
- * Reload Welt2000 data file. Can be called after a configuration change.
+ * Starts a thread, which is loading the requested Welt2000 data.
+ */
+void MapContents::loadWelt2000DataViaThread()
+{
+  QMutexLocker locker( &welt2000Mutex );
+
+  _globalMapView->slot_info( tr("loading Welt2000") );
+
+  Welt2000Thread *w2000Thread = new Welt2000Thread( this );
+
+  // Register a special data type for return results. That must be
+  // done to transfer the results between different threads.
+  qRegisterMetaType<AirfieldListPtr>("AirfieldListPtr");
+
+  // Connect the receiver of the results. It is located in this
+  // thread and not in the new opened thread.
+  connect( w2000Thread,
+           SIGNAL(loadedLists( bool,
+                               QList<Airfield>*,
+                               QList<Airfield>*,
+                               QList<Airfield>*  )),
+           this,
+           SLOT(slotWelt2000LoadFinished( bool,
+                                          QList<Airfield>*,
+                                          QList<Airfield>*,
+                                          QList<Airfield>* )) );
+  w2000Thread->start();
+}
+
+/**
+ * This slot is called by the Welt2000 load thread to signal, that the
+ * requested airfield data have been loaded. The passed lists must be
+ * deleted in this method.
+ */
+void MapContents::slotWelt2000LoadFinished( bool ok,
+                                            QList<Airfield>* airfieldListIn,
+                                            QList<Airfield>* gliderfieldListIn,
+                                            QList<Airfield>* outlandingListIn )
+{
+  QMutexLocker locker( &welt2000Mutex );
+
+  if( ok == false )
+    {
+      qDebug() << "slotWelt2000LoadFinished: Welt2000 loading failed!";
+
+      _globalMapView->slot_info( tr("Welt2000 load failed") );
+
+      delete airfieldListIn;
+      delete gliderfieldListIn;
+      delete outlandingListIn;
+      return;
+    }
+
+  // Take over the new loaded lists. The passed lists must be deleted!
+  airfieldList = *airfieldListIn;
+  delete airfieldListIn;
+
+  gliderfieldList = *gliderfieldListIn;
+  delete gliderfieldListIn;
+
+  outLandingList = *outlandingListIn;
+  delete outlandingListIn;
+
+  _globalMapView->slot_info( tr("Welt2000 loaded") );
+
+  emit mapDataReloaded();
+}
+
+#endif
+
+/**
+ * Reload the Welt2000 data file. Can be called after a configuration change or
+ * a file download.
  */
 void MapContents::slotReloadWelt2000Data()
 {
+#ifndef WELT2000_THREAD
+
   // @AP: defined a static mutex variable, to prevent the recursive
   // calling of this method
   static bool mutex = false;
@@ -2342,15 +2560,15 @@ void MapContents::slotReloadWelt2000Data()
   GpsNmea::gps->enableReceiving( false );
 
   airfieldList.clear();
-  gliderSiteList.clear();
+  gliderfieldList.clear();
   outLandingList.clear();
 
-  _globalMapView->message( tr("Reloading Welt2000 started") );
+  _globalMapView->slot_info( tr("loading Welt2000") );
 
   Welt2000 welt2000;
-  welt2000.load( airfieldList, gliderSiteList, outLandingList );
+  welt2000.load( airfieldList, gliderfieldList, outLandingList );
 
-  _globalMapView->message( tr("Reloading Welt2000 finished") );
+  _globalMapView->slot_info( tr("Welt2000 loaded") );
 
   emit mapDataReloaded();
 
@@ -2359,6 +2577,13 @@ void MapContents::slotReloadWelt2000Data()
   GpsNmea::gps->enableReceiving( true );
 
   mutex = false; // unlock mutex
+
+#else
+
+  // Reload Welt2000 data in an extra thread.
+  loadWelt2000DataViaThread();
+
+#endif
 }
 
 /**
@@ -2388,7 +2613,10 @@ void MapContents::slotReloadAirspaceData()
   qDeleteAll(airspaceList);
   airspaceList.clear();
 
-  _globalMapView->message( tr("Reloading Airspaces started") );
+  // free all internal allocated memory in QList
+  airspaceList = SortableAirspaceList();
+
+  _globalMapView->slot_info( tr("loading Airspaces") );
 
   OpenAirParser oap;
   oap.load( airspaceList );
@@ -2396,7 +2624,7 @@ void MapContents::slotReloadAirspaceData()
   // finally, sort the airspaces
   airspaceList.sort();
 
-  _globalMapView->message( tr("Reloading Airspaces finished") );
+  _globalMapView->slot_info( tr("Airspaces loaded") );
 
   emit mapDataReloaded();
 
@@ -2442,17 +2670,17 @@ void MapContents::drawList( QPainter* targetP,
 
       break;
 
-    case GliderSiteList:
+    case GliderfieldList:
       //list="GliderList";
-      //len=gliderSiteList.count();
+      //len=gliderfieldList.count();
       showProgress2WaitScreen( tr("Drawing glider sites") );
 
-      for (int i = 0; i < gliderSiteList.size(); i++)
+      for (int i = 0; i < gliderfieldList.size(); i++)
         {
-          if( gliderSiteList[i].drawMapElement(targetP) && showAfLabels )
+          if( gliderfieldList[i].drawMapElement(targetP) && showAfLabels )
             {
               // required and draw object is appended to the list
-              drawnAfList.append( &gliderSiteList[i] );
+              drawnAfList.append( &gliderfieldList[i] );
             }
         }
 
@@ -2482,8 +2710,10 @@ void MapContents::drawList( QPainter* targetP,
   // qDebug( "List=%s, Length=%d, drawTime=%dms", list, len, t.elapsed() );
 }
 
-void MapContents::drawList(QPainter* targetP, unsigned int listID)
-  {
+void MapContents::drawList( QPainter* targetP,
+                            unsigned int listID,
+                            QList<BaseMapElement *>& drawnElements )
+{
   //const char *list="";
   //uint len = 0;
 
@@ -2504,14 +2734,14 @@ void MapContents::drawList(QPainter* targetP, unsigned int listID)
 
       break;
 
-    case GliderSiteList:
+    case GliderfieldList:
       //list="GliderList";
-      //len=gliderSiteList.count();
+      //len=gliderfieldList.count();
       showProgress2WaitScreen( tr("Drawing glider sites") );
 
-      for (int i = 0; i < gliderSiteList.size(); i++)
+      for (int i = 0; i < gliderfieldList.size(); i++)
         {
-          gliderSiteList[i].drawMapElement(targetP);
+          gliderfieldList[i].drawMapElement(targetP);
         }
 
       break;
@@ -2565,7 +2795,13 @@ void MapContents::drawList(QPainter* targetP, unsigned int listID)
       //len=cityList.count();
       showProgress2WaitScreen( tr("Drawing cities") );
       for (int i = 0; i < cityList.size(); i++)
-        cityList[i].drawMapElement(targetP);
+        {
+          if( cityList[i].drawMapElement(targetP) )
+            {
+              drawnElements.append( &cityList[i] );
+            }
+        }
+
       break;
 
     case VillageList:
@@ -2584,12 +2820,12 @@ void MapContents::drawList(QPainter* targetP, unsigned int listID)
         landmarkList[i].drawMapElement(targetP);
       break;
 
-    case HighwayList:
-      //list="HighwayList";
-      //len=highwayList.count();
-      showProgress2WaitScreen( tr("Drawing highways") );
-      for (int i = 0; i < highwayList.size(); i++)
-        highwayList[i].drawMapElement(targetP);
+    case MotorwayList:
+      //list="MotorwayList";
+      //len=MotorwayList.count();
+      showProgress2WaitScreen( tr("Drawing motorways") );
+      for (int i = 0; i < motorwayList.size(); i++)
+        motorwayList[i].drawMapElement(targetP);
       break;
 
     case RoadList:
@@ -2690,6 +2926,8 @@ void MapContents::drawIsoList(QPainter* targetP)
       count++; // yes
     }
 
+  int elevationIndexOffest = GeneralConfig::instance()->getElevationColorOffset();
+
   QMap< int, QList<Isohypse> >* isoMaps[2] = { &groundMap, &terrainMap };
 
   for( int i = 0; i < count; i++ )
@@ -2715,9 +2953,9 @@ void MapContents::drawIsoList(QPainter* targetP)
 
           const QList<Isohypse> &isoList = it.value();
 
-          for (int i = 0; i < isoList.size(); i++)
+         for (int j = 0; j < isoList.size(); j++)
             {
-              Isohypse isoLine = isoList.at(i);
+              Isohypse isoLine = isoList.at(j);
 
               if( drawTerrain )
                 {
@@ -2726,6 +2964,27 @@ void MapContents::drawIsoList(QPainter* targetP)
                   // normally with an offset of one.
                   int colorIdx = isoLine.getElevationIndex();
 
+                  // We can move the color index by a user configuration option
+                  // to get a better color schema.
+                  if( elevationIndexOffest != 0 && i == 1 )
+                    {
+                      int newIndex = colorIdx + elevationIndexOffest;
+
+                      if( newIndex >= 0 && newIndex <= SIZEOF_TERRAIN_COLORS )
+                        {
+                          // Move color index to the new position
+                          colorIdx = newIndex;
+                        }
+                      else if( newIndex < 0 )
+                        {
+                          colorIdx = 0;
+                        }
+                      else if( newIndex >=  SIZEOF_TERRAIN_COLORS )
+                        {
+                          colorIdx = SIZEOF_TERRAIN_COLORS - 1;
+                        }
+                    }
+
                   targetP->setBrush( QBrush(conf->getTerrainColor(colorIdx),
                                      Qt::SolidPattern));
                 }
@@ -2897,7 +3156,7 @@ bool MapContents::isInWaypointList(const QPoint& wgsCoord)
 {
   for (int i=0; i < wpList.count(); i++)
     {
-      const wayPoint& wpItem = wpList.at(i);
+      const Waypoint& wpItem = wpList.at(i);
 
       if ( wgsCoord == wpItem.origP )
         {
@@ -2916,7 +3175,7 @@ bool MapContents::isInWaypointList(const QString& name )
 {
   for (int i=0; i < wpList.count(); i++)
     {
-      const wayPoint& wpItem = wpList.at(i);
+      const Waypoint& wpItem = wpList.at(i);
 
       if ( name == wpItem.name )
         {
@@ -2937,7 +3196,7 @@ unsigned short MapContents::countNameInWaypointList( const QString& name )
 
   for (int i=0; i < wpList.count(); i++)
     {
-      const wayPoint& wpItem = wpList.at(i);
+      const Waypoint& wpItem = wpList.at(i);
 
       if ( name == wpItem.name )
         {
