@@ -6,12 +6,12 @@
 **
 ************************************************************************
 **
-**   Copyright (c):  2009-2011 by Axel Pauli
+**   Copyright (c):  2009-2012 by Axel Pauli
 **
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: settingspageterraincolors.h 4718 2011-12-08 19:24:23Z axel $
+**   $Id: settingspageterraincolors.h 5010 2012-03-16 12:44:07Z axel $
 **
 ***********************************************************************/
 
@@ -28,7 +28,7 @@
  *
  * \date 2002-2011
  *
- * \version $Id: settingspageterraincolors.h 4718 2011-12-08 19:24:23Z axel $
+ * \version $Id: settingspageterraincolors.h 5010 2012-03-16 12:44:07Z axel $
  *
  */
 
@@ -58,6 +58,12 @@ public:
 
   virtual ~SettingsPageTerrainColors();
 
+protected:
+
+  virtual void showEvent(QShowEvent *);
+
+  virtual void hideEvent(QHideEvent *);
+
 public slots:
 
   /**
@@ -123,6 +129,9 @@ private:
 
   // provides an positive or negative elevation offset index.
   QSpinBox* elevationOffset;
+
+  /** Auto sip flag storage. */
+  bool m_autoSip;
 };
 
 #endif
