@@ -12,7 +12,7 @@
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: windanalyser.h 4502 2010-12-09 22:32:02Z axel $
+ **   $Id: windanalyser.h 4741 2011-12-23 14:28:14Z axel $
  **
  ***********************************************************************/
 
@@ -63,7 +63,7 @@ public slots:
   /**
    * Called if the flight mode changes
    */
-  void slot_newFlightMode( Calculator::flightmode newMode );
+  void slot_newFlightMode( Calculator::FlightMode newMode );
 
   /**
    * Called if a new sample is available in the sample list.
